
#include "router.h"
#include "syslog.h"

//#define ENABLE_ROUTING_SYSLOG


// This buffer is used to copy data from CSX to CSX
BYTE buffer[BLOCK_SIZE];  

// The configuration data is referenced using the intf and route pointers.
// The raw data is in one big character array.  These pointers give shape to the array.
fw_interface *intf;
fw_route *route;
fw_syslog *syslog_server;

// This stores the number of route entries.
BYTE number_of_routes;

// This stoes the context information.
CSX_CONTEXT context[NOI];



static struct ROUTE_CACHE route_cache; 

void route_cache_init()
  {
    memset((void*)&route_cache, 0, sizeof(struct ROUTE_CACHE));
  }

void route_cache_add(BYTE CSX)
  {
    BYTE i = 0;

    route_cache.pointer[CSX] = (++route_cache.pointer[CSX]) % ROUTE_CACHE_LENGTH;
    i = route_cache.pointer[CSX];

    route_cache.csx[CSX].entry[i].mac_src = context[CSX].mac_src;
    route_cache.csx[CSX].entry[i].mac_dst = context[CSX].mac_dst;
    route_cache.csx[CSX].entry[i].CSX_dst = context[CSX].CSX_dst;    
  }

BOOL route_cache_exec(BYTE CSX)
  {
    BYTE i;
    
    for (i=0u; i<ROUTE_CACHE_LENGTH; i++)
      {
        if (route_cache.csx[CSX].entry[i].mac_src.IPAddr.Val == context[CSX].header.SourceAddress.Val)
          {
            if (route_cache.csx[CSX].entry[i].mac_dst.IPAddr.Val == context[CSX].header.DestAddress.Val)
              {
                LED_C_ON;
                route_packet(route_cache.csx[CSX].entry[i].CSX_dst, 
                             CSX, 
                            &route_cache.csx[CSX].entry[i].mac_dst, 
                            &route_cache.csx[CSX].entry[i].mac_src, 
                            &context[CSX].header, 
                             context[CSX].payloadlen);
                return TRUE;
              }
          }
      }
    return FALSE;
  }





/* The set_route function is my first stab at routing algorithms.
 * The only comment I can make is that surely there is a better method.
 */

BOOL set_route(BYTE CSX, NODE_INFO *dst, NODE_INFO *src)
  {
    BYTE i;
    int r = -1;
    BYTE out_csx;
    
    /* A router MUST generate a Time Exceeded message Code 0 (In Transit)
     * when it discards a packet due to an expired TTL field.  A router MAY
     * have a per-interface option to disable origination of these messages
     * on that interface, but that option MUST default to allowing the
     * messages to be originated.
     *  ....
     * When a router forwards a packet, it MUST reduce the TTL by at least
     * one.  If it holds a packet for more than one second, it MAY decrement
     * the TTL by one for each second.
     *
     * If the TTL is reduced to zero (or less), the packet MUST be
     * discarded, and if the destination is not a multicast address the
     * router MUST send an ICMP Time Exceeded message, 
     */
    if (context[CSX].header.TimeToLive < 2ul) return FALSE;  // NOTE: Need ICMP message.
    
    // examine the router table for interfaces having specifc routes.
    // The routes should come ordered with the "longest match" first
    for (i=0u; i<number_of_routes; i++)
      {  
        if ((context[CSX].header.DestAddress.Val & route[i].mask.Val) == route[i].net.Val)
          {
            r = i;
            break;
          }
      }
      
    // return false if we did not find a route to the host
    // NOTE: We need an ICMP message here
    if (r<0) return FALSE; 

    out_csx = route[r].ino;
    
    if (CSX == out_csx)
      {
        // In this case, we will need to route the packet out the same interface
        // it came in.  But, we cannot do this yet.
        // Let's drop this packet.  I think the RFC requires we pass it along, but
        // we need a place to store it before we can arp on that network. 
        return FALSE;
      }
          
    // The packet is sent from "CSX" to "route[r].ino"
    // 1. I am the router for this network
    // 2. I am on the same network as this remote ip
    // 3. I will have to forward to a remote router
    
    if (route[r].ip.Val == intf[out_csx].ip.Val)
      {
        // I am the route for this network!
        // this means, I will arp for the destination host on the local subnet
        // dst[CSX] is already defined this way
      }
    else if ((intf[out_csx].ip.Val & intf[out_csx].mask.Val) == (context[CSX].header.DestAddress.Val & intf[out_csx].mask.Val))
      {
        // Is the interface on a local network.
        // I am on the same local network as this host.  So, I will arp for the actual host.
        // dst[CSX] is already defined this way
      }
    else
      {
        // the route is on the network.  We will have to arp for the remote router address.
        dst->IPAddr.Val = route[r].ip.Val;
      }
      
    // clear out the existing dst mac address
    memset((void*) &dst->MACAddr, 0, sizeof(MAC_ADDR));  
      
    // change the MAC address of the source to my interface's mac address 
    memcpy((void*) &src->MACAddr, (void*)intf[out_csx].mac, sizeof(MAC_ADDR));
     
    context[CSX].CSX_dst = out_csx; // save the CSX destination for this route
    return TRUE;
  }
  
  
/* The route_packet function sets the IP header and 
 * simply copyies data from CSX_src to CSX_dst
 */
 
void route_packet(BYTE CSX_dst,
                 BYTE CSX_src,
                 NODE_INFO *mac_dest,
                 NODE_INFO *mac_source,
                 IP_HEADER *header,
                 WORD len) // length of the data in the IP packet
  {
    unsigned int block;
    BYTE bklog;
    
    LED_D_ON;
    
#if defined (ENABLE_ROUTING_SYSLOG)    

    bklog = getPktCount(CSX_src);
    sprintf(SYSLOG_PTR,"picsw: Routing %d bytes from %d to %d, ID=%x (backlog=%d)",
        len,CSX_src,CSX_dst,header->Identification,bklog);
#endif        
    
	  while(!MACIsTxReady(CSX_dst));
	  MACSetWritePtr(CSX_dst,BASE_TX_ADDR);
	
    IPPutHeader2(CSX_dst,  
                 &(mac_dest->MACAddr),    // MAC address of packet
                 &(mac_source->MACAddr),  // MAC address of packet
                 &(header->DestAddress),  //IP address
                 &(header->SourceAddress),  //IP address
                 header->Protocol,
                 header->Identification,
                 header->FragmentInfo,
                 header->TimeToLive - 1,  // TTL -1 
                 len);
    

    
    block = BLOCK_SIZE; 
    while (len > 0ul)
      {
        if (len < ((unsigned long)block)) 
          block=((WORD)len);    
          
        MACGetArray(CSX_src, buffer, block);  
        
        //if (linked[CSX_dst_0])                       
        MACPutArray(CSX_dst, buffer, block);
        
        len = len - block;             
      }  
    MACDiscardRx(CSX_src);    
    MACFlush(CSX_dst);
    
#if defined (ENABLE_ROUTING_SYSLOG)
    syslogit();
#endif    
    
  }



// Most of these checks are in accordance with RFC 1812.
BOOL validate_address(BYTE CSX, IP_HEADER *header)
  {
        
	  // is  packet destined for a general broadcast address?  
	  // we choose not to respond to broadcast requests      
    if (IS_BCAST_1(header->DestAddress.Val))
      return FALSE;      
	    
	  // is the packet destined for a directed broadcast address?    
	  // we choose not to resond to directed broadcast        			              
    if (IS_LTD_BCAST_1(header->DestAddress.Val, intf[CSX].ip.Val, intf[CSX].mask.Val))
      return FALSE;
    
    
    /*  Incoming datagrams with a source address of { 0, 0 } which are
        received for local delivery (see Section [5.2.3]), MUST be
        accepted if the router implements the associated protocol and
        that protocol clearly defines appropriate action to be taken.
        Otherwise, a router MUST silently discard any locally-delivered
        datagram whose source address is { 0, 0 }.  */
                  
    if (IS_BCAST_0(header->SourceAddress.Val))
      return FALSE;

      
    /*  SHOULD silently discard on receipt (i.e., do not even deliver to
        applications in the router) any packet addressed to 0.0.0.0 or {
        <Network-prefix>, 0 }.  If these packets are not silently
        discarded, they MUST be treated as IP broadcasts (see Section
        [5.3.5]).  There MAY be a configuration option to allow receipt
        of these packets.  This option SHOULD default to discarding
        them.  */
        
    if (IS_BCAST_0(header->DestAddress.Val))
      return FALSE;

    if (IS_LTD_BCAST_0(header->DestAddress.Val, intf[CSX].ip.Val, intf[CSX].mask.Val))
      return FALSE;
  
    /*   { -1, -1 } Limited broadcast.  It MUST NOT be used as a source address.
         A datagram with this destination address will be received by
         every host and router on the connected physical network, but
         will not be forwarded outside that network.  
         
         { <Network-prefix>, -1 } Directed Broadcast - a broadcast directed to the 
         specified network prefix.  It MUST NOT be used as a source address. */
    
    if (IS_BCAST_1(header->SourceAddress.Val))
      return FALSE;   

    if (IS_LTD_BCAST_1(header->SourceAddress.Val, intf[CSX].ip.Val, intf[CSX].mask.Val))
      return FALSE;      

    /*   { 127, <any> }  Internal host loopback address.  Addresses of this form MUST
         NOT appear outside a host.  */
         
    if (IS_LOOPBACK(header->SourceAddress.Val))
      return FALSE;

    if (IS_LOOPBACK(header->DestAddress.Val))
      return FALSE;      
      

// NOTE: Add check for IS_MCAST
    
    return TRUE;
  }
  
// This fucntion converts a sequence of HEX values into an IP address unsigned long value.
void intf_hex_to_ip(IP_ADDR *ip)
  {
    ip->Val =  ( ( (unsigned long) ip->v[0] ) 
               | ( (unsigned long) ip->v[1] )<<8ul 
               | ( (unsigned long) ip->v[2] )<<16ul
               | ( (unsigned long) ip->v[3] )<<24ul);
  } 
  
void initilize_rule_data()
  {
    BYTE i;
    
    syslog_server = (fw_syslog*) SYSLOG_SERVER;
    intf_hex_to_ip(&(syslog_server->ip));
            
    intf = (fw_interface*) INTERFACES;
    for (i=0u; i<NOI; i++)
      {
        // convert the 4 bytes of hex data to a 4 byte unsigned long value.
        intf_hex_to_ip(&(intf[i].mask));
        intf_hex_to_ip(&intf[i].ip);
      }

    route = (fw_route*) ROUTES;
    
    // The number of routes is determined based on the size of the ROUTES array.
    number_of_routes = (sizeof(ROUTES)/sizeof(fw_route));
    for (i=0u; i<NOI; i++)
      {
        intf_hex_to_ip(&route[i].mask);
        intf_hex_to_ip(&route[i].net);
        intf_hex_to_ip(&route[i].ip);
      }
  }
  
