package FW::IP;

use strict;

use base qw(FW::Base);

use FW::CIDR;

sub new
  {
    my $pkg = shift;
    my $text = shift;
 
    my $obj = $pkg->SUPER::new($text);
    $obj->{'ANY'} = 0;
    $obj->{'NUMBER'} = 0;

    bless $obj, $pkg;

    $obj->expand($text) if ($text);

    return $obj;
  }

sub getAny
  {
    my $obj = shift;
    return $obj->{'ANY'};
  }

sub setAny
  {
    my $obj = shift;
    $obj->{'ANY'} = shift;
  }

sub expand
  {
    my $obj = shift;
    my $addr = shift;

    my ($ip,$bits) = ($addr =~ /(\d+\.\d+\.\d+\.\d+)\/(\d+)/);

    $obj->setBits($bits);
    $obj->setIP($ip);

    my $mask = FW::CIDR::cidr2mask( $obj->getBits() );
    $obj->setMask($mask);

    my @ip = split(/\./,$obj->getIP());
    my @mask = split(/\./,$obj->getMask());

    # apply CIDR mask to ip to eliminate extraneous precission
    for (my $i=0; $i<4; $i++)
      { 
        $ip[$i] = unpack('C1',pack('C1',$ip[$i]) & pack('C1',$mask[$i])); 
      }

    $ip = join(".",@ip);
    $mask = join(".",@mask);

    my @hex = (@ip,@mask);
    for (my $i=0; $i<8; $i++)
      { 
        $hex[$i] = sprintf("%.2x",$hex[$i]);
      }

    $obj->setHex( join(" ",@hex) );
    $obj->setHexLen( 8 );
    $obj->setCIDR( $ip . "/" . $bits );
    $obj->setNET( $ip . "/" . $mask );
  }

sub getNET
  {
    my $obj = shift;
    return $obj->{'NET'};
  }

sub setNET
  {
    my $obj = shift;
    $obj->{'NET'} = shift;
  }

sub getCIDR
  {
    my $obj = shift;
    return $obj->{'CIDR'};
  }

sub setCIDR
  {
    my $obj = shift;
    $obj->{'CIDR'} = shift;
  }


sub getMask
  {
    my $obj = shift;
    return $obj->{'MASK'};
  }

sub setMask
  {
    my $obj = shift;
    $obj->{'MASK'} = shift;
  }


sub getIP
  {
    my $obj = shift;
    return $obj->{'IP'};
  }

sub setIP
  {
    my $obj = shift;
    $obj->{'IP'} = shift;
  }


sub getBits
  {
    my $obj = shift;
    return $obj->{'BITS'};
  }

sub setBits
  {
    my $obj = shift;
    $obj->{'BITS'} = shift;
  }

return 1;
